"""
    Cataloging of file system files
    
    Copyright 2006 Mikko Ohtamaa <miohtama@mail.student.oulu.fi>

"""

__author__  = 'Mikko Ohtamaa <snaapy@gmail.com>'
__docformat__ = 'restructuredtext'

# Python imports
import os
import string
import re
from types import StringType, UnicodeType

# Zope imports
from DateTime import *
import zLOG

# Plone imports
from Products.Archetypes.ExtensibleMetadata import FLOOR_DATE, CEILING_DATE
from Products.CMFCore.CMFCorePermissions import ModifyPortalContent

# Local imports
from CatalogProxy import CatalogProxy
from FileFilter import FileFilter
from util import makeRelativePath, makePortalPath

class FileCataloger:
    """ Catalog file system files 
    
    Add file system files to portal_catalog so that navtree and breadcrumbs 
    work for file system files.
    """
    
    def __init__(self, portal, view_roles, uid_base, base_path, file_filter, catalog_text):
        """ Initialize cataloger 
        
        @param catalog_text Catalog file contents (not just paths)
        @param portal portal object
        @param view_roles roles allowed to view files
        @param uid_base portal path where file_path is mounted
        @param file_path filesystem absolute path to the file
        @param base_path PloneLocalFolderNG file system mount point
        @param filter FileFilter used to define files going for cataloging
        """
        self.portal = portal
        self.catalog_tool = portal.portal_catalog
        self.mimetypes_tool = portal.mimetypes_registry       
        self.view_roles = view_roles
        self.base_path = base_path
        self.catalog_text = catalog_text
        
        if "//" in uid_base:
            # ensure double slash doesn't happen 
            # causes path cataloging go haywire 
            raise ValueError, "Double slashes in PloneLocalFolderNG path"
        
        self.uid_base = uid_base
            
        # Cache control timestamps
        self.effective = FLOOR_DATE
        self.expires = CEILING_DATE
        
        self.file_suffixes_skipList=['.metadata', '.plfngtmp']
        
        self.files_cataloged = 0
        self.skip_invalid_ids = True
        self.file_filter = file_filter
    
    def catalogFile(self, fs_path):
        """ Catalog one file 
        
        Recursively go to subdirectores
        
        Assume initial path is not invalid or contain invalid
        characters for Plone ids-
        
        @param fs_path full path file (or folder) to catalog        
        @return number of files cataloged
        """
           
        if not os.path.exists(fs_path):
            raise "Path doesn't exist " + fs_path
                         
        # relative file system path from mount point to this item 
        relative_path = makeRelativePath(fs_path, self.base_path)
        #print "path:" + str(makePortalPath(self.uid_base, relative_path))
             
        mi = self.mimetypes_tool.classify(data=None, filename=fs_path.lower())
        try:
           item_mime_type = mi.normalized()
        except:
           item_mime_type = self.mimetypes_tool.defaultMimetype
 
        # Use mimetype to determine icon 
        if os.path.isdir(fs_path):
            icon_path = "folder_icon.gif"
        else:
            try:
                icon_path = mi.icon_path
            except:
                icon_path = 'unknown.png'
                
        uid = self.uid_base + "/" + relative_path
        #print "uid:" + uid
                
        #print "id:" + os.path.basename(fs_path)
        #print "relative_path:" + relative_path

        proxy_object = CatalogProxy(id=os.path.basename(fs_path),
                                 path=makePortalPath(self.uid_base, relative_path),
                                 icon_path = icon_path,
                                 mime_type = item_mime_type,
                                 encoding = None,                                 
                                 effective = self.effective,
                                 expires = self.expires,
                                 view_roles = self.view_roles,
                                 is_folderish = os.path.isdir(fs_path),
                                 catalog_text = self.catalog_text,
                                 )

        # don't catalog root folder
        # (as it's represented by PloneLocalFolderNG instance and
        #  is cataloged by Archetypes mechanism)    
        if relative_path != "" and relative_path != "/":                      
            self.catalog_tool.catalog_object(proxy_object, uid)               
            self.files_cataloged += 1    
       
        if os.path.isdir(fs_path):
            subfiles = self.getFiles(fs_path)
            #print "uid:" + uid + " files:" + str(subfiles)
                   
            for file in subfiles:
                # If fs_path is a directory, recurse to it
                self.catalogFile(file)
            
        return self.files_cataloged
                
    def getFiles(self, fs_path):
        """ Return available filenames in path 
        
        If fs_path is not a folder, return empty list
        
        Listing is filtered against FileFilter rule
        
        @param path File system full path to a directory
        @return list of files for cataloging with full paths
        """
  
        filteredFileList = []
  
        files= os.listdir(fs_path)
        
        result = []

        for item in files:
            full_path = os.path.join(fs_path, item)
            if self.file_filter.isValidFile(item):
                result.append(full_path)

        
        return result
    