"""
    
    File filtering functions

    Copyright 2006 Mikko Ohtamaa <miohtama@mail.student.oulu.fi>
    
    Based on PloneLocalFolderNG.util code

"""

# Python imports
import os, re

# Zope imports
import zLOG

bad_id=re.compile(r'[^a-zA-Z0-9-_~,.$\(\)# ]').search


class FileFilter:
    """ Filter files based on filename masking rules
    """
    
    def __init__(self, mimetypesTool, skipInvalidIds,  
                       filetypePhrasesSkipList, filePrefixesSkipList,
                       fileSuffixesSkipList, fileNamesSkipList,
                       folderPrefixesSkipList, folderSuffixesSkipList,
                       folderNamesSkipList):
        """ Construct FileFilter
        
        Various filter lists are given as parameters """
           
        self.mimetypesTool = mimetypesTool
        self.skipInvalidIds = skipInvalidIds
        self.filetypePhrasesSkipList = filetypePhrasesSkipList
        self.filePrefixesSkipList = filePrefixesSkipList
        self.fileSuffixesSkipList = fileSuffixesSkipList
        self.fileNamesSkipList = fileNamesSkipList
        self.folderPrefixesSkipList = folderPrefixesSkipList
        self.folderSuffixesSkipList = folderSuffixesSkipList
        self.folderNamesSkipList = folderNamesSkipList

    def isValidFile(self, item):
        """ Check filename against filtering rules 
        
        @param item basename of file/folder (no path)

        """
        
        skipThisItem = False
        
        # note checkValidId special return format
        if self.skipInvalidIds and checkValidId(item) != True:
            return False                    
        
        if os.path.isdir(item):
             
             for prefix in self.folderPrefixesSkipList:
                if item.startswith(prefix): 
                   skipThisItem = 1
                   break
             
             for suffix in self.folderSuffixesSkipList:
                if item.endswith(suffix): 
                   skipThisItem = 1
                   break
             
             for completeName in self.folderNamesSkipList:
                if item == completeName: 
                   skipThisItem = 1
                   break
             
             return not skipThisItem
         
        else:
             
             if self.mimetypesTool:
                mi = self.mimetypesTool.classify(data=None, filename=item.lower())
                try:
                   item_mime_type = mi.normalized()
                except:
                   item_mime_type = self.mimetypesTool.defaultMimetype
                   #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
                   # "warning: no filename mime type for %s" % filename)
             
                for filetypePhrase in self.filetypePhrasesSkipList:
                   if item_mime_type.find(filetypePhrase) >= 0:
                      skipThisItem=1
                      break
             
             
             for prefix in self.filePrefixesSkipList:
                if item.startswith(prefix): 
                   skipThisItem = 1
                   break
             
             #print "skiplist:" + str(self.fileSuffixesSkipList)
             for suffix in self.fileSuffixesSkipList:
                if item.endswith(suffix): 
                   skipThisItem = 1
                   break
             
             for completeName in self.fileNamesSkipList:
                if item == completeName: 
                   skipThisItem = 1
                   break
             
             return not skipThisItem
             
    def listFolder(self, folder_path, separate_files_and_folders = True, inverse = False):
        """ List folder contents, filtering against the rules
        
        @param folder_path 
            absolute fs folder path
        
        @param separate_files_and_folders
            Backward compatible way to list files and folders
            in different sequencies.
               
        @param inverse 
            If true, list filtered out files instead files passed filtering
        
        @return 
            If separate_files_and_folders == true
                tuple (file_list, folder_list) which passed filtering
            else 
                sequence [files + folders]
        """
        
        filteredFileList = []
        filteredFolderList = []
       
        if not os.path.exists(folder_path):
           # raise exception here?
           zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
           "getFilteredFSItems() :: FSfullPath not found (%s)" % folder_path )
          
        else:   
            try:
                rawItemList = os.listdir(folder_path)
            except:
                zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
                "getFilteredFSItems() :: error reading folder (%s) contents" \
                % folder_path )
             
                if separate_files_and_folders:
                    return filteredFileList, filteredFolderList
                else:
                    return []
    
            for item in rawItemList:
                FSfullPathItemName = os.path.join(folder_path, item)
                 
                if os.path.isdir(FSfullPathItemName):                 
                    if self.isValidFile(item) ^ inverse: 
                        filteredFolderList.append(item)         
                else:            
                    if self.isValidFile(item) ^ inverse:
                        filteredFileList.append(item)

       
        if separate_files_and_folders:
            # return tuple(files, folders)
            return filteredFileList, filteredFolderList
        else:
            # return seq containing files and folders
            return filteredFileList + filteredFolderList


def checkValidId(id):
    """ This is essentially checkValidId() from OFS/OBjectManager.py """
    
    if not id:
        return 'The id is invalid because it is empty or not specified'
    if bad_id(id) is not None:
        return \
        'The id is invalid because it contains characters illegal in URLs.'
    if id in ('.', '..'):
        return 'The id is invalid because it is not traversable.'
    if id.startswith('_'):
        return 'The id is invalid because it begins with an underscore.'
    if id.startswith('aq_'):
        return 'The id is invalid because it begins with "aq_".'
    if id.endswith('__'):
        return 'The id is invalid because it ends with two underscores.'
    if id == 'REQUEST':
        return 'The id is invalid because REQUEST is a reserved name.'
    if '/' in id:
        return 'The id is invalid because it contains "/" characters illegal in URLs.'
    return True                    