"""

    PloneLocalFolderNG
    
    Local file system proxy for Plone
    
    Copyright 2006 Andreas Jung, Mike Garnsey, Red Innovation


"""

__author__  = 'Mikko Ohtamaa <mikko@redinnovation.com>'
__docformat__ = 'restructuredtext'

# Python imports
import os
import shutil
import sys
from string import split,find,rstrip

# Zope imports
from DateTime.DateTime import DateTime
from AccessControl import ClassSecurityInfo
from AccessControl.Permission import Permission
from Acquisition import aq_chain
from OFS.ObjectManager import checkValidId
import zLOG

try:
    from ZPublisher.Iterators import filestream_iterator
    STREAM_SUPPORT=1
except ImportError:
    STREAM_SUPPORT=0


# Plone imports
try:
    from Products.Archetypes.public import BaseSchema, Schema
    from Products.Archetypes.public import SelectionWidget
    from Products.Archetypes.public import StringField, StringWidget
    from Products.Archetypes.public import BooleanField, BooleanWidget
    from Products.Archetypes.public import BaseContent, registerType
except:
    from Products.Archetypes.ArchetypeTool import registerType
    from Products.Archetypes.BaseContent import BaseContent
    from Products.Archetypes.BaseContent import BaseSchema
    from Products.Archetypes.Field import BooleanField, StringField, TextField
    from Products.Archetypes.Schema import Schema
    from Products.Archetypes.Storage import AttributeStorage
    from Products.Archetypes.Widget import BooleanWidget, SelectionWidget, StringWidget


from Products.CMFCore.CMFCorePermissions import *
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.PloneTool import PloneTool

# Local imports
from FileProxy import FileProxy
from FileCataloger import FileCataloger
from FileFilter import FileFilter
from FileFilter import checkValidId
from onFileAccess import _onFileAccess
from util import determine_bytes_usage 
from util import getMetadataElement, setMetadata
from util import setZipInfoMetadata, upzipFile
from util import zipFiles
from util import _isFileDeletable, _makeFileDeletable
from util import generate_md5
from util import makeRelativePath
from util import makePortalPath
from util import getFolderProperties
from config import *


tempFSpath = os.path.join(INSTANCE_HOME,'Products','PloneLocalFolderNG',
                          'tests','SamplesFiles')+os.path.sep
if not os.path.exists(tempFSpath):
	tempFSpath = os.path.join(SOFTWARE_HOME,'Products','PloneLocalFolderNG',
	                          'tests','SamplesFiles')+os.path.sep

schema = BaseSchema +  Schema((
    StringField('folder',
                default=tempFSpath,
                validators=("isValidExistingFolderPath",),
                write_permission=CMFCorePermissions.ManagePortal,
                required=1,
                widget=StringWidget(label='Local directory name')
                ),
    StringField('default_page',
                write_permission=CMFCorePermissions.ManagePortal,
                default='',
                widget=StringWidget(label='filename of default page to view',
                                    description='specify the filename of the \
                                    default page to show (if file exists in \
                                    current folder) instead of current folder \
                                    contents for folder View action.  Leave \
                                    this empty to disable this default page \
                                    feature.',)
                ),
    BooleanField ('require_MD5_with_upload',
                write_permission=CMFCorePermissions.ManagePortal,
                default=0,
                widget=BooleanWidget(
                        label='Require MD5 with file upload?',
                        description='Require user to provide MD5 checksum \
                        along with file being uploaded.',
                        ),
                ),
    BooleanField ('generate_MD5_after_upload',
                write_permission=CMFCorePermissions.ManagePortal,
                default=1,
                widget=BooleanWidget(
                        label='Generate MD5 after upload?',
                        description='Generate an MD5 checksum for a file \
                        immediately after it is uploaded.',
                        ),
                 ),
    StringField('external_syscall_md5',
                default='none',
                validators=("isValidExternalMD5Utility",),
                write_permission=CMFCorePermissions.ManagePortal,
                required=1,
                widget=StringWidget(label='external md5 system call',
                                    description='To use the standard python \
                                    md5 implementation, specify "none".',
                       )
                ),
    StringField('quota_maxbytes',
                write_permission=CMFCorePermissions.ManagePortal,
                default='0',
                widget=StringWidget(label='max # bytes quota limit',
                                    description='Prevent users from \
                                    performing actions that would violate \
                                    quota limits as follows: -1 -> use \
                                    parent-level quota_maxbytes; 0 -> no \
                                    limit, n -> # bytes quota limit',)
                ),
    BooleanField ('allow_file_unpacking',
                write_permission=CMFCorePermissions.ManagePortal,
                default=1,
                widget=BooleanWidget(
                        label='Allow File Unpacking?',
                        description='Allow users to extract the contents of \
                        archive files (eg, .zip, .tar) on to server local \
                        filesystem.',
                        ),
                ),
    BooleanField ('overwrite_existing_files',
                write_permission=CMFCorePermissions.ManagePortal,
                default=1,
                widget=BooleanWidget(
                        label='Allow Existing Files to be Overwritten?',
                        description='Allow existing files to be overwritten \
                        when files are added.',
                        ),
                ),            
    StringField('hidden_file_prefixes',
                write_permission=CMFCorePermissions.ManagePortal,
                default='.',
                widget=StringWidget(label='hide files with these prefixes',
                                    description='this field contains the \
                                    comma-separated list of filename prefixes \
                                    used to determine which files to hide',)
                ),
    StringField('hidden_file_suffixes',
                write_permission=CMFCorePermissions.ManagePortal,
                default='.metadata,.plfngtmp',
                widget=StringWidget(label='hide files with these suffixes',
                                    description='this field contains the \
                                    comma-separated list of filename suffixes \
                                    used to determine which files to hide',)
                ),
    StringField('hidden_file_names',
                write_permission=CMFCorePermissions.ManagePortal,
                default='',
                widget=StringWidget(label='hide files with these exact names',
                                    description='this field contains the comma\
                                    -separated list of filenames to hide',)
                ),
    StringField('hidden_folder_prefixes',
                write_permission=CMFCorePermissions.ManagePortal,
                default='.',
                widget=StringWidget(label='hide folders with these prefixes',
                                    description='this field contains the \
                                    comma-separated list of folder prefixes \
                                    used to determine which files to hide',)
                ),
    StringField('hidden_folder_suffixes',
                write_permission=CMFCorePermissions.ManagePortal,
                default='.metadata,.plfngtmp',
                widget=StringWidget(label='hide folders with these suffixes',
                                    description='this field contains the \
                                    comma-separated list of folder suffixes \
                                    used to determine which files to hide',)
                ),
    StringField('hidden_folder_names',
                write_permission=CMFCorePermissions.ManagePortal,
                default='CVS',
                widget=StringWidget(label='hide folders with these exact names',
                                    description='this field contains the \
                                    comma-separated list of folders to hide',)
                ),
    BooleanField ('cataloging_enabled',
                write_permission=CMFCorePermissions.ManagePortal,
                default=1,
                widget=BooleanWidget(
                        label='Is file cataloging enabled?',
                        description='This field controls whether or not files \
                        can be cataloged.',
                        ),
                ),
    StringField('filetypes_not_to_catalog',
                write_permission=CMFCorePermissions.ManagePortal,
                default='image/,video/,audio/,/zip,/x-tar',
                widget=StringWidget(label='filetypes for catalog action to skip',
                                    description='this comma-separated list of \
                                    (partial) mimetype phrases is used by the \
                                    catalog action to determine which files \
                                    NOT to catalog',)
                ),
    StringField('folder_address_display_style',
               default='currentIdOnly',
               required=1,
               vocabulary='getAddressDisplayStyleVocab',
               widget=SelectionWidget(
                  label='Folder Address Display Style',
                  description="""Select the folder address style to be \
                  displayed""",
                  format="select",),
                ),
    BooleanField ('fileBackup_enabled',
                write_permission=CMFCorePermissions.ManagePortal,
                default=0,
                widget=BooleanWidget(
                        label='Enable file backup on upload overwrite?',
                        description='Enable this field to archive files \
                        (to the backup-folder path) that will be overwritten \
                        by uploaded files.',
                        ),
                ),
    StringField('backup_folder',
                validators=("isValidExistingFolderPath",),
                write_permission=CMFCorePermissions.ManagePortal,
                required=0,
                widget=StringWidget(label='Local backup directory name')
                ),
    BooleanField ('ignore_readonly',
                write_permission=CMFCorePermissions.ManagePortal,
                default=0,
                widget=BooleanWidget(
                        label='Ignore OS read-only attribute settings',
                        description='Enable this field to bypass any \
                        os read-only attribute settings when performing \
                        operations on files',
                        ),
                ),
    StringField('item_move_rules',
               default='allow_local_movement_only',
               required=1,
               vocabulary='getMoveActionRestrictionsVocab',
               widget=SelectionWidget(
                  label='Move action restrictions for PLFNG contents',
                  description="""Select the move action restrictions for \
                  files/folders contained by this PLFNG instance """,
                  format="select",),
                ),
    BooleanField ('log_file_accesses',
                write_permission=CMFCorePermissions.ManagePortal,
                default=0,
                widget=BooleanWidget(
                        label='log file accesses?',
                        description='Enable this field to invoke the file \
                        access logging mechanism in _onFileAccess()',
                        ),
                ),
    BooleanField ('count_file_accesses',
                write_permission=CMFCorePermissions.ManagePortal,
                default=0,
                widget=BooleanWidget(
                        label='count file accesses?',
                        description='Enbable this field to invoke the file \
                        access counting mechanism in _onFileAccess()',
                        ),
                ),
   BooleanField ('catalog_text',
                write_permission=CMFCorePermissions.ManagePortal,
                default=True,
                widget=BooleanWidget(
                        label="Make file content searchable",
                        description="When cataloging, read all files and add their content to portal search index",
                        ),
                ),                               
    ))




class PloneLocalFolderNG(BaseContent):
    """ Local file system access
    
  
    
    """
    
    # Folderish object
    isPrincipiaFolderish = True
    
    content_icon = 'folder_icon.gif'
    
    schema = schema
    
    allowed_content_types = []
    
    security = ClassSecurityInfo()

    actions = (
      { 'id': 'view',
        'name': 'View',
        'action': 'string:${object_url}/plfng_view',
        'permissions': (View,),
        },
      { 'id': 'folderlisting',
        'name': 'Folder Listing',
        'action': 'string:${object_url}/plfng_view',
        'permissions': (View,),
        'visible'       : 0
        },
      { 'id': 'edit',
        'name': 'Edit',
        'action': 'string:$object_url/base_edit',
        'permissions': (ManagePortal,),
        },
      { 'id': 'catalog',
        'name': 'Catalog',
        'action': 'string:$object_url/plfng_catalog',
        'permissions': (ManagePortal,),
        },
      { 'id': 'diagnostics',
        'name': 'Diagnostics',
        'action': 'string:$object_url/plfng_summary',
        'permissions': (ManagePortal,),
        },
      { 'id': 'metadata',
        'name': 'Properties',
        'action': 'string:${object_url}/base_metadata',
        'permissions': (ManageProperties,),
        },
      )

    def __bobo_traverse__(self, REQUEST, name, RESPONSE=None):
        """ Make this container traversable
        
        Store wanted file system path parts in not-so-friendly-named
        REQUEST["_e"] list.                 
        """
        
        if name == 'folder_contents': 
            name = ''
    
        if not REQUEST.has_key('_e'):
            REQUEST['_e'] = []
        
        try:
            
            # figure out file system path for this part of traverse
            base = self._getFSBasePath()

            #print "fsbase: " + base            
            destpath = base
            for already_visited_path_part in REQUEST['_e']:
                destpath += os.sep + already_visited_path_part
            
            #print "current:" + os.sep + name
            destpath += os.sep + name
        
            #print "destpath:" + destpath
        
            if os.path.exists(destpath):
                REQUEST['_e'].append(name)
                return self
            else:
                try: 
                    return getattr(self, name)
                except AttributeError:           
                     # missing attribute look ups would fill error log
                     # zLOG.LOG('PLFNG', zLOG.ERROR , "probably missing path in PloneLocalFolderNG:" + destpath)
                     pass
                # Should not raise NotFoundError on __bobo_traverse__.
                # Instead, just return None and let
                # ZPublisher+Acquisition do it's job                
                return
			            
        except: 
            # don't interrupt traversing if some exception is raised
            # should not happen
            zLOG.LOG('PLFNG', zLOG.ERROR , "exception in __bobo_traverse__", error=sys.exc_info())
            return	

    def __call__(self, REQUEST=None, RESPONSE=None, *args, **kw):
        """
        Invokes a particular action on an item contained by the PLFNG instance.
        The item to take action on is identified via the '_e' field of 
        self.REQUEST.  The action to take is identified via the 'action'
        field of self.REQUEST.

        :Parameters:
            - `REQUEST`: the standard HTML request data structure.
            - `RESPONSE`: the standard HTML response data structure.
            - `\*args`: unused by this method.
            - `\*\*kw`: unused by this method.

        :Return:
            - refer to following table:

        +-----------+----------------------+----------------------------------+
        | item type |REQUEST.get('action') | return value                     |
        +===========+======================+==================================+
        |           | 'catalog'            | self.catalogFSContent()           |
        |           +----------------------+----------------------------------+
        |           | 'deleteFile'         | present results of deleteFile    |
        |           |                      | action via portal_status_message |
        |           |                      | 'plfng_view' redirect on file's  |
        |           |                      | parent                           |
        |   file    +----------------------+----------------------------------+
        |           | 'editMetadata'       | 'plfng_editMetadata' redirect on |
        |           |                      | file's parent                    |
        |           +----------------------+----------------------------------+
        |           | 'unpack'             | present results of deleteFile    |
        |           |                      | action via portal_status_message |
        |           |                      | 'plfng_view' redirect on file's  |
        |           |                      | parent                           |
        |           +----------------------+----------------------------------+
        |           |  None                | redirect to showFile() on file   |
        +-----------+----------------------+----------------------------------+
        |           | 'deleteFolder'       | present results of deleteFile    |
        |           |                      | action via portal_status_message |
        |           |                      | 'plfng_view' redirect on folder's|
        |           |                      | parent                           |
        |           +----------------------+----------------------------------+
        |           |                      | If 'default_page' field for the  |
        | subfolder |                      | PLFNG object is set, return      |
        |           |                      | showFile() on the file           |
        |           | None                 | by the subfolder)                |
        |           |                      | with the name specified in       |
        |           |                      | 'default_page'.  Otherwise,      |
        |           |                      | 'plfng_view' redirect on the     |
        |           |                      | subfolder.                       | 
        +-----------+----------------------+----------------------------------+
        |           |                      | if REQUEST == None :             |
        |           |                      |   return None                    |
        | the base  | n/a                  | else :                           |
        | PLFNG obj |                      |   'plfng_view' redirect to self  |
        |           |                      |   redirect to self               |
        +-----------+----------------------+----------------------------------+


        """
        #zLOG.LOG('PLFNG', zLOG.INFO , "__call__() :: REQUEST = %s" % REQUEST)

        if REQUEST == None:
            return None

        else:
           destpath = self._getFSFullPath(PLFNGRelativePath='')

           trimmedFSBasePath = self._getFSBasePath()
           rel_dir = destpath.replace(trimmedFSBasePath, '')
           rel_dir = rel_dir.replace('\\','/')
           if rel_dir.startswith('/'): rel_dir = rel_dir[1:]
           
           print "__Call__"
           print "destpath:" + destpath
           print "basepath:" + trimmedFSBasePath
           print "rel_dir:" + rel_dir
           print "req:" + str(REQUEST)
           print "action:" + REQUEST.get("action", "")
                          
           if not rel_dir:
               # We are being visited directly. Redirect to
               # plfng_view here to avoid infinite redirection below.
                              
               if hasattr(self, "default_page") and self.default_page:
                   FSDefaultPageFullPath = \
                     os.path.join(destpath, self.default_page)
                   if os.path.exists(FSDefaultPageFullPath):
                       return self.showFile(FSDefaultPageFullPath,
                                            REQUEST,
                                            RESPONSE)           
               
               
               #if rel_dir = '':
               #    # if we don't do this absolute_url double adds our id
               #    dest = self.absolute_url(relative=True) + "/plfng_view"    
               #else:
               dest = self.absolute_url(relative=False) + "/plfng_view"
               
               print "redirecting to:" + dest
               RESPONSE.redirect(dest)
               return
           

           if os.path.isfile(destpath):
               #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO ,
               #          "__call__() :: %s :: isfile()" % destpath)
               if REQUEST.get('action', '') == 'unpack':
                  self.unpackFile(os.path.dirname(rel_dir),
                                                  destpath)

               elif REQUEST.get('action', '') == 'deleteFile':
                  self.deleteFile(destpath, REQUEST, RESPONSE)

               elif REQUEST.get('action', '') == 'catalog':
                  #catalogTool = getToolByName(self, 'portal_catalog')
                  return self.catalogFSContent()
               elif REQUEST.get('action', '') == 'editMetadata':
                   dest = self.absolute_url(True) + '/plfng_editMetadata'
                   RESPONSE.redirect(dest)
               else:
                  return self.showFile(destpath, REQUEST, RESPONSE)
           elif os.path.isdir(destpath):
               if REQUEST.get('action', '') == 'deleteFolder':
                   #msg = 'listFolderContents() :: deleteFolder(%s)' % destpath
                   #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , msg)
                   redir_dest = self.absolute_url(True) + '/plfng_view'
                   if self.deleteFolder(destpath):
                      msg = '?portal_status_message='+destpath+' deleted.'
                      RESPONSE.redirect(redir_dest + msg)
                      return
                   else:
                      msg = '?portal_status_message='+destpath+' NOT deleted.'
                      RESPONSE.redirect(redir_dest + msg)
                      return
          
               if hasattr(self, "default_page") and self.default_page:
                   FSDefaultPageFullPath = \
                     os.path.join(destpath, self.default_page)
                   if os.path.exists(FSDefaultPageFullPath):
                       return self.showFile(FSDefaultPageFullPath,
                                            REQUEST,
                                            RESPONSE)

               # Visiting a folder directly may also cause
               # infinite redirection.
               #print " xview at " + self.absolute_url(relative=False) + '/plfng_view'
               RESPONSE.redirect(self.absolute_url(relative=False) + '/plfng_view')
               return

    def _createProxy(self, id, **kw):
        """ Create FileObject proxies for content listing
    
        
        """
        
        from os.path import join, dirname, exists
        destination = kw['destination']
        proxy_type = kw.get('type', 'content')
        properties = kw.get('properties', None)
        rel_dir = kw['rel_dir']

        fullpath = join(destination, id).replace('\\','/')
        foldername = dirname(fullpath)
        mi = self.mimetypes_registry.classify(data=None, filename=id.lower())

        proxy = FileProxy(id, filepath=fullpath, fullname=fullpath,
                          properties=properties)
        proxy.setAbsoluteURL(self.absolute_url() + "/" + id)

        if proxy_type == 'folder':
            proxy.setIconPath('folder_icon.gif')
            proxy.setMimeType('folder')
        else:
            try:
               item_mime_type = mi.normalized()
            except:
               item_mime_type = self.mimetypes_registry.defaultMimetype
            try:
               iconPath = mi.icon_path
            except:
               iconPath = 'unknown.png'   

            proxy.setIconPath(iconPath)
            proxy.setMimeType(item_mime_type)

        try:
             proxy.setComment(getMetadataElement(fullpath,
                                                 section="GENERAL",
                                                 option="comment",
                                                 REQUEST=self.REQUEST))
        except:
             proxy.setComment('')
        try:
             proxy.setTitle(getMetadataElement(fullpath,
                                               section="GENERAL",
                                               option="title",
                                               REQUEST=self.REQUEST))
        except:
             proxy.setTitle('')

        try:
             proxy.setChecksum(getMetadataElement(fullpath,
                                                  section="DIAGNOSTICS",
                                                  option="md5",
                                                  REQUEST=self.REQUEST))
        except:
             proxy.setChecksum('')

        return proxy.__of__(self)

    def _getAttribute(self, attrname):
        """ method will return a list """
        attr = getattr(self, attrname, None)
        if attr is not None:
            return attr.split(',')
        return []

    def _getFSBasePath(self):
        """ returns the base File System path of the item. """

        PLFNGFolderFSFullPath = self.getFolder()
                
        if PLFNGFolderFSFullPath == '': self.setFolder(tempFSpath)

        if PLFNGFolderFSFullPath == '':
           request = self.REQUEST
           msg = 'local directory field of PLFNG instance not set!'
           redirURL = self.absolute_url() + '/base_edit?portal_status_message='+msg
           #zLOG.LOG('PLFNG', zLOG.INFO , "_getFSBasePath(): redir = "+ redirURL)
           #zLOG.LOG('PLFNG', zLOG.INFO , "_getFSBasePath(): request = %s" % request)
           request.RESPONSE.redirect( redirURL )
           # MG: not sure why, but the preceding redirect only works from ZMI,
           # so throw the following exception in case the redirect fails
           msg=msg + ' (use base_edit to fix)'
           raise ValueError(msg)

        trimmedFSBasePath = os.path.normpath(PLFNGFolderFSFullPath)

        if os.path.exists(trimmedFSBasePath):
            #msg = '_getFSBasePath() :: FSpath = ' + FSFullPath
            #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , msg)
            return trimmedFSBasePath
        else:
            raise ValueError('non-existent path: %s' % trimmedFSBasePath)


    def _getFSFullPath(self, PLFNGRelativePath=''):
        """ get the full file system path for the item contained by this PLFNG
            instance as specified by the relative path for the item via the
            PLFNGRelativePath parameter if provided or the '_e' field of 
            self.REQUEST

            Note: this method & _getFSBasePath() are the 'security gatekeeper'
            methods that B{all} PLFNG methods B{must} use to to obtain the
            actual file sytem path PLFNG instance contents.
            
            @param self: the PLFNG object instance
            @param PLFNGRelativePath: the URL path (relative to the PLFNG
            instance 'self') of the item contained by the PLFNG instance 
            for which the full filesystem path is desired.
            @return: the full file system path for this PLFNG item
        """

        trimmedFSBasePath = self._getFSBasePath()

        if not PLFNGRelativePath:
            PLFNGRelativePath = '/'.join(self.REQUEST.get('_e', []))

        # protect against potential '_e' shenanigans
        if PLFNGRelativePath.startswith('/') or \
          PLFNGRelativePath.find('..') > -1:
            msg = "illegal relative path: " + PLFNGRelativePath
            zLOG.LOG('PLFNG', zLOG.INFO , "validFolder() :: " + msg)
            raise ValueError(msg)
                
        FSFullPath = \
         os.path.normpath(os.path.join(trimmedFSBasePath,PLFNGRelativePath))

        if not FSFullPath.startswith(trimmedFSBasePath):
            zLOG.LOG('PLFNG', zLOG.INFO , "_getFSFullPath() :: \
             FSFullPath = %s, trimmedFSBasePath = %s" % \
             (FSFullPath,trimmedFSBasePath))
            raise ValueError('illegal path: %s' % FSFullPath)

        if os.path.exists(FSFullPath):
            #msg = '_getFSFullPath() :: FSpath = ' + FSFullPath
            #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , msg)
            return FSFullPath
        else:
            msg = "_getFSFullPath() :: path bad for: "+ FSFullPath
            zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , msg)
            raise ValueError('illegal path: %s' % FSFullPath)
            
    security.declarePrivate('createFileFilter')
    def createFileFilter(self):
        """ Create FileFilter object based on user entered filtering field values
        """
        
        return FileFilter(
            mimetypesTool = getToolByName(self, 'mimetypes_registry'),
            skipInvalidIds=True,                                          
            filetypePhrasesSkipList=[],
            filePrefixesSkipList = self._getAttribute('hidden_file_prefixes'),
            fileSuffixesSkipList = self._getAttribute('hidden_file_suffixes'),
            fileNamesSkipList = self._getAttribute('hidden_file_names'),
            folderPrefixesSkipList = self._getAttribute('hidden_folder_prefixes'),
            folderSuffixesSkipList = self._getAttribute('hidden_folder_suffixes'),
            folderNamesSkipList = self._getAttribute('hidden_folder_names'))

    security.declareProtected(AddPortalContent, 'addFile')
    def addFile(self, sourceFSfullPath, destPLFNGrelativePath='',
                destBasename=None, moveFile=0, inputMD5=None):
        """ 
        add a file on the Plone site`s filesystem to the file space of the
        PLFNG instance.  This method supports a variety of enhanced PLFNG
        features, including file space quota limit support, file backup & 
        revisioning support, and MD5-based file integrity testing.  It also
        screens for illegal destination file and path names.
        
        :Parameters:
            - `sourceFSfullPath`: the full path name of the file on the 
              filesystem that is to be added to the PLFNG instance.
            - `destPLFNGrelativePath`: the relative path within the PLFNG 
              instance that the file is to be added to.  If the does not exist,
              it will be created.
            - `destBasename`: (default=None) the optional name for the newly
              added file.  If not provided, the name of the added file will be
              the same as the source file name.
            - `moveFile`: (default=0) flag to control whether the source file
              is to be copied (moveFile=0) or moved (moveFile=1) to the file
              space of the PLFNG instance.
            - `inputMD5`: (default=None) the optional MD5 checksum, in 32 
              character long hexadecimal format, for the file that is to be
              added to the PLFNG instance.  If provided, it must match the 
              server-generated MD5 checksum for the file in order for file to
              be added to the PLFNG instance.  Note, this mainly to support
              the file uploads via HTMLFormUploadFile() with MD5 validation.
            

        :Return:
            - refer to following table:
            
        +------------------------------------------------------+--------------+
        | condition / result                                   | return value |
        +======================================================+==============+
        | PLFNG instance not configured with valid FS folder   |      0       |
        +------------------------------------------------------+--------------+
        | sourceFSfullPath parameter == ''                     |      0       |
        +------------------------------------------------------+--------------+
        | destination file name NOT valid                      |      0       |
        +------------------------------------------------------+--------------+
        | destPLFNGrelativePath NOT valid                      |      0       |
        +------------------------------------------------------+--------------+
        | destPLFNGrelativePath NOT valid                      |      0       |
        +------------------------------------------------------+--------------+
        | destPLFNGrelativePath NOT valid                      |      0       |
        +------------------------------------------------------+--------------+
        | destPLFNGrelativePath NOT valid                      |      0       |
        +------------------------------------------------------+--------------+
        | destPLFNGrelativePath NOT valid                      |      0       |
        +------------------------------------------------------+--------------+
 
        """    
        
        #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO, "addFile(%s,%s,%s,%s,%s)" % \
        #(sourceFSfullPath,destPLFNGrelativePath,destBasename,moveFile,inputMD5))

        # 1st, make sure that the PLFNG instance is configured with
        # a valid FS folder field value

        if self.validFolder() != 1:
           msg = "addFile() :: Error! validFolder() test failed."
           zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , msg )
           return 0

        # make sure that a file was specified

        if not os.path.isfile(sourceFSfullPath):
           zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "addFile() :: Error! \
           sourceFSfullPath = %s not a file" % sourceFSfullPath )
           return 0

        if not destBasename:
           destBasename = os.path.basename(sourceFSfullPath)

        # make sure destination file name is valid

        checkValidIdResult = checkValidId(destBasename)
        if checkValidIdResult != True:
           zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "addFile() :: \
           Error! destination filename (%s) is not permitted: %s" % \
           (destBasename,checkValidIdResult))
           return 0

        # make sure the PLFNG destination relative path is valid

        if destPLFNGrelativePath.startswith('/'):
           zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "addFile() :: \
           Error! PLFNG destination path (%s) cannot begin with forward \
           slash" % destPLFNGrelativePath)
           return 0

        destPLFNGrelativePath = rstrip(destPLFNGrelativePath,"/")
        pathItemList = split(destPLFNGrelativePath, "/")
        if not destPLFNGrelativePath == '':
           for pathItem in pathItemList:
              checkValidIdResult = checkValidId(pathItem)
              if checkValidIdResult != 1:
                 zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "addFile() :: \
                 Error! PLFNG destination path (%s) invalid: %s" % \
                 (destPLFNGrelativePath,checkValidIdResult))
                 return 0

        # make sure add operation will not violate quota limit restrictions

        max_allowed_bytes = self.getAvailableQuotaSpace()
        try: contentLength = os.stat(sourceFSfullPath)[6]
        except: contentLength = 0
        if max_allowed_bytes != -1 and contentLength > max_allowed_bytes:
           # uploaded file rejected as it would result in quota limit
           # being exceeded
           zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "addFile() :: \
           file rejected! CONTENT_LENGTH (%s) + usedBytes(%s) > \
           max_allowed_bytes(%s)" % \
           (contentLength,usedBytes, max_allowed_bytes) )
           return 0

        # if a MD5 checksum was provided, make sure that the
        # server-generated MD5 matches it

        serverMD5 = None
        if inputMD5:
           serverMD5 = generate_md5(sourceFSfullPath,self.external_syscall_md5)
           if inputMD5 != serverMD5:
              zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "addFile() :: \
              file rejected! inputMD5 (%s) != generated MD5 (%s)" % \
              (inputMD5,serverMD5) )
              return 0

        # create destination path as necessary

        FSBasePath = self._getFSBasePath()

        destFSfullPathFolderName = \
          os.path.join(FSBasePath, destPLFNGrelativePath)
          
        #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "addFile() :: \
        #  FSBasePath=%s, destFSfullPathFolderName=%s" % (FSBasePath, destFSfullPathFolderName) )  

        if not os.path.exists(destFSfullPathFolderName):
           try:
              os.makedirs(destFSfullPathFolderName)
           except:
              msg = "addFile() :: Error creating destination path: " + \
                destFSfullPathFolderName
              zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , msg)
              return 0

        # backup existing file as necessary

        destFSfullPath = os.path.join(destFSfullPathFolderName, destBasename)

        if not os.path.exists(destFSfullPath):
           newRevisionNumber = 1

        else:
           # get revision of existing file (or 1 if revision metadata missing)
           oldRevisionNumberText = getMetadataElement(destFSfullPath,
                                                      section="GENERAL",
                                                      option="revision",
                                                      REQUEST=self.REQUEST)
           if oldRevisionNumberText:
              oldRevisionNumber = int(oldRevisionNumberText)
           else:
              oldRevisionNumber = 1

           newRevisionNumber = oldRevisionNumber + 1

           # backup existing file if file backup is enabled & \
           # backup_folder path is set
           if self.fileBackup_enabled and self.backup_folder:
              backupdestpath = \
                os.path.join(self.backup_folder, destPLFNGrelativePath)

              if not oldRevisionNumberText:
                 setMetadata(destFSfullPath,
                             section="GENERAL",
                             option="revision",
                             value=oldRevisionNumber)

              # move existing file to backup location, changing its name
              # at the backup location to include trailing rev.#
              backupFileSuffix = '.' + str(oldRevisionNumber)
              backupfilename = os.path.join(backupdestpath, destBasename) + \
                               backupFileSuffix
              # create skeleton dir structure under backupFolder if necessary
              if not os.path.exists(backupdestpath):
                 os.makedirs(backupdestpath)
              shutil.move(destFSfullPath, backupfilename)

        # copy/move file to destination path
        tmsg = ''

        try:
           if moveFile==1:
              #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "addFile() :: \
              #move(%s,%s)" % (sourceFSfullPath, destFSfullPath) )

              # just in case the file being added is already contained
              # by this PLFNG instance, perform a shutil.copy() then
              # PLFNG.deleteFile() on it to handle metadata and backup stuff 
              trimmedFSBasePath = self._getFSBasePath() 
              if sourceFSfullPath.startswith(trimmedFSBasePath): 
                  #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "addFile() :: \
                  #shutil.copy(%s,%s)" % (sourceFSfullPath, destFSfullPath) )
                  shutil.copy(sourceFSfullPath, destFSfullPath)
                  #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "addFile() :: \
                  #deleteFile(destpath=%s)" % sourceFSfullPath )
                  result = self.deleteFile(fileToDelete=sourceFSfullPath,
                                           REQUEST=None,
                                           RESPONSE=None)
                  #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "addFile() :: \
                  #deleteFile(destpath=%s) returned %d" % (sourceFSfullPath,result) )
              else:
                  tmsg = '(not local) shutil.move'
                  #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "addFile() :: \
                  #(not local) shutil.move(%s,%s)" % (sourceFSfullPath, destFSfullPath) )
                  #shutil.move(sourceFSfullPath, destFSfullPath)
           else:
              #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "addFile() :: \
              #shutil.copy(%s,%s)" % (sourceFSfullPath, destFSfullPath) )
              shutil.copy(sourceFSfullPath, destFSfullPath)
        except:
           zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "addFile() :: \
           Error! could not add file: %s" % tmsg)
           return 0

        return newRevisionNumber

    security.declareProtected(ManagePortal, 'catalogFiles')
    def catalogFiles(self, PLFNGRelativePath=''):
        """ Catalog files inside this PloneLocalFolderNG
        
        """
        portal = getToolByName(self, 'portal_url')
        portalId = portal.getPortalObject().getId()
        
        #print "catalogFiles:"+ str('/'+portalId+'/'+portal.getRelativeContentURL(self))
        
        file_filter = self.createFileFilter()

        cataloger = FileCataloger(
            portal = portal,
            uid_base = str('/'+portalId+'/'+portal.getRelativeContentURL(self)),
            view_roles = Permission(View,'',self).getRoles(),
            base_path = self._getFSBasePath(),
            file_filter=file_filter,
            catalog_text = self.catalog_text,
            )
                  
        result = cataloger.catalogFile(self._getFSFullPath(PLFNGRelativePath))      
        
        # recook mime_icons.css.dtml so we have 
        # new possible icon types available for navtree_macro        
        rr_css=getToolByName(self,'portal_css')
        rr_css.cookResources()
                    
        return result
        
    def getPhysicalPath(self):
        """ Return portal root relative path for this item
        
        getPhysicalPath() is overriden to support mounted
        file system object paths correctly for breadcrumbs
        and navtree.
        
        You don't believe how hard was to get this method and
        related catalogs right. If I had to redo it I'd rather
        go into a cage with an angry polar bear. Pfft. 16+ hours work 
        here in seemingly few lines.
        """
        path = BaseContent.getPhysicalPath(self)
        #print "physpath:" + str(path)
        REQUEST = self.REQUEST       
                
        rel_path = makeRelativePath(self._getFSFullPath(PLFNGRelativePath=''), self._getFSBasePath())
                
        # Add file system relative path to base url
        path = path + tuple(rel_path.split("/"))
        
        # filter empty items from path
        # (caused by leading and trailing / and other weird artifacts)
        # NOTE: looks like CMFCore/URLTool relies on behavior that 
        # there is empty '' in the beginning of the path
        path = ("",) + filter(lambda x: x != '', path)
        # print "getPhys:" + str(path)
        return path
        
        
    security.declareProtected('View', 'contentValues')
    def contentValues(self, spec=None, filter=None, sort_on=None, reverse=0):
        """ CMFPlone.PloneFolder.PloneFolder.contentValues """
        filteredFileList = []
    
        destpath = self._getFSFullPath(PLFNGRelativePath='')

        trimmedFSBasePath = self._getFSBasePath()
        rel_dir = destpath.replace(trimmedFSBasePath, '')

        if rel_dir.startswith('/'): rel_dir = rel_dir[1:]

        fileFilter = self.createFileFilter()
        
        filteredFileList = fileFilter.listFolder(folder_path=destpath,
                                                 separate_files_and_folders=False)


        proxies = []
        for file in filteredFileList:
            _proxy = self._createProxy(file,
                                       rel_dir=rel_dir,
                                       destination=destpath)
            proxies.append(_proxy)

        return proxies


 
    security.declareProtected(AddPortalContent, 'create_directory')
    def create_directory(self, dirname, REQUEST):
        """ create a sub-directory """
        dirname = dirname.replace('\\','/')
        if dirname.startswith('/') or dirname.find('..') > -1 or \
           dirname.find(':') > -1:
            msg="create directory aborted: illegal directory name: " + dirname
            zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , msg )
            if REQUEST:
               REQUEST.RESPONSE.redirect(REQUEST['URL1']+\
                  '/plfng_view?portal_status_message=' + msg)
            return 0

        FSBasePath = self._getFSFullPath(PLFNGRelativePath='')
        destpath = os.path.join(FSBasePath, dirname)

        #print destpath
        if os.path.exists(destpath):
            raise ValueError('Directory %s already exists' % dirname)

        # try..except to avoid exposing the realworld path
        try:
            os.makedirs(destpath)
        except:
            raise RuntimeError('Directory could not be created')

        redir_dest = self.absolute_url(True) + '/plfng_view'
        msg = '?portal_status_message='+destpath+' created.'        
        if REQUEST: 
            REQUEST.RESPONSE.redirect(redir_dest + msg)
        
        


    security.declareProtected('Delete objects', 'deleteFile')
    def deleteFile(self, fileToDelete, REQUEST, RESPONSE):
        """ delete the file """

        if not os.path.exists(fileToDelete):
           return 0

        trimmedFSBasePath = self._getFSBasePath() 
        
        # only allow files contained by this PLFNG instance to be deleted.
        if not fileToDelete.startswith(trimmedFSBasePath):
            #msg = 'deleteFile() :: %s not contained by this PLFNG instance" \
            #% fileToDelete
            #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , msg)        
            return 0
            
        okToDeleteFile = 0

        # determine the delete permission setting for the file
        fileHasDeletePermission = _isFileDeletable(fileToDelete)
        if fileHasDeletePermission:
           okToDeleteFile = 1

        # if we normally wouldnt be able to delete this file 
        # but ignore_readonly is set, then change its permissions
        elif not fileHasDeletePermission and getattr(self, "ignore_readonly", 0):
           _makeFileDeletable(fileToDelete)
           
        if okToDeleteFile:
           # move file to backupFolder if file backup is enabled &
           # backup_folder path is set
           if self.fileBackup_enabled and self.backup_folder:
               # get revision of existing file (or 1 if rev. metadata missing)
               oldRevisionNumberText = \
                 getMetadataElement(fileToDelete,
                                    section="GENERAL",
                                    option="revision",
                                    REQUEST=self.REQUEST)
               if oldRevisionNumberText:
                   oldRevisionNumber = int(oldRevisionNumberText)
               else:
                   oldRevisionNumber = 1
                   # uncomment the next statement to update the file's
                   # metadata if its also going to be backed up
                   #setMetadata(fileToDelete, section="GENERAL", \
                   #  option="revision", value=oldRevisionNumber)
   
               # move existing file to backup location, changing its name
               # at the backup location to include trailing rev.#
               rel_dir = fileToDelete.replace(trimmedFSBasePath, '')
               rel_dir = rel_dir.replace('\\','/')
               if rel_dir.startswith('/'): rel_dir = rel_dir[1:]
               backupdestpath = \
                 os.path.dirname(os.path.join(self.backup_folder,rel_dir))
               backupFileSuffix = '.' + str(oldRevisionNumber)
               backupfilename = \
                 os.path.join(backupdestpath, os.path.basename(fileToDelete))\
                   + backupFileSuffix
               # create skeleton dir structure under backupFolder if necessary
               if not os.path.exists(backupdestpath):
                   os.makedirs(backupdestpath)
               try:
                   shutil.move(fileToDelete, backupfilename)
                   msg = fileToDelete + ' deleted.'
               except:
                   msg = fileToDelete + ' NOT deleted (OS error).'
           else:
               
               try:
                  os.remove(fileToDelete)
                  msg = fileToDelete + ' deleted.'
                  fileWasDeleted = 1
               except:
                  msg = fileToDelete + ' NOT deleted (OS error).'

           # delete the associated .metadata file, too!
           metadataFileToDelete = fileToDelete + '.metadata'
           if os.path.exists(metadataFileToDelete):
               os.remove(metadataFileToDelete)
        else:             
            msg = fileToDelete + ' NOT deleted (it is Read-Only).' 

        #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO, "deleteFile()return msg = %s" % msg)
        
        if REQUEST and RESPONSE:
           RESPONSE.redirect(REQUEST['URL1']+\
             '/plfng_view?portal_status_message=' + msg)
 
        return 1


    security.declareProtected('Delete objects', 'deleteFolder')
    def deleteFolder(self, folderToDelete):
        """ delete the folder """
        
        FSBasePath = self._getFSBasePath()
        if not folderToDelete.startswith(FSBasePath):
           msg = "illegal folder: " + folderToDelete
           zLOG.LOG('PLFNG', zLOG.INFO , "deleteFolder() :: " + msg)
           raise ValueError(msg)
        try:
           shutil.rmtree(folderToDelete)
           # delete the associated .metadata file, too!
           folderName = os.path.basename(folderToDelete)
           metadataFileToDelete = os.path.join(os.path.dirname(folderToDelete),folderName) + '.metadata'
           if os.path.exists(metadataFileToDelete):
               os.remove(metadataFileToDelete)
           return 1
        except:
           return 0   

    security.declareProtected(CMFCorePermissions.AccessContentsInformation,
                              'folderlistingFolderContents')
    def folderlistingFolderContents(self, spec=None, contentFilter=None,
                                    suppressHiddenFiles=0 ):
        """
        Calls listFolderContents in protected only by ACI so that
        folder_listing can work without the List folder contents permission,
        as in CMFDefault
        """
        return self.listFolderContents(spec,contentFilter,suppressHiddenFiles)


    security.declareProtected(CMFCorePermissions.View,
                              'getAddressDisplayStyleVocab')
    def getAddressDisplayStyleVocab(self):
        """
        vocabulary method for 'folder_address_display_style' attribute
        """
        return DisplayList(
           (
            ('PLFNG_Base_Relative','PLFNG_Base_Relative'),
            ('PLFNG_TitleLinks_Relative','PLFNG_TitleLinks_Relative'),
           ))


    security.declareProtected('View', 'getAvailableQuotaSpace')
    def getAvailableQuotaSpace(self):
        """ get the remaining space in # of bytes that are available for
            storage by the PLFNG instance without violating its byte quantity
            quota limit. A positive or zero return value indicates remaining
            space in # of bytes. A negative return value indicates that
            parent-level quota_maxbytes attribute is to be used to determine
            the remaining space in # of bytes """

        # if quota_maxbytes for PLFNG instance > 0  :: use its value

        # if quota_maxbytes for PLFNG instance = 0  :: return -1
        #   (quota limit checking disabled)

        # if quota_maxbytes for PLFNG instance < 0 ::
        #   traverse up the acquisition tree looking for first container with
        #   a non-zero 'quota_maxbytes' attribute.  If such a container is
        # found, find out the total number of bytes used by the contents of
        # this container

        PLFNGInstanceQuotaLimit = int(self.quota_maxbytes)

        if PLFNGInstanceQuotaLimit > 0:
           quotaLimit = int(self.quota_maxbytes)
           folderProps = self.getProperties()
           usedBytes = folderProps['size']
           max_allowed_bytes = quotaLimit - usedBytes
           if max_allowed_bytes < 0:
              max_allowed_bytes = 0

        elif PLFNGInstanceQuotaLimit == 0:
           max_allowed_bytes = -1

        else:
           quotaLimit = 0
           for parent in self.aq_chain[1:]:
              if hasattr(parent, "quota_maxbytes"):
                 quotaLimit = int(getattr(parent, "quota_maxbytes"))
                 if quotaLimit > 0:
                    usedBytes = determine_bytes_usage(parent)
                    break
           if quotaLimit > 0:
              max_allowed_bytes = quotaLimit - usedBytes
              if max_allowed_bytes < 0:
                 max_allowed_bytes = 0
           else:
              max_allowed_bytes = -1

        return max_allowed_bytes

    security.declareProtected('View', 'getFileInfo')
    def getFileInfo(self, PLFNGRelativePath, arg):
        """
        get info for a PLFNG file
        
        :Parameters:
            - PLFNGRelativePath

        :Return:
            - the requested info for the PLFNG file
        """

        result = ''

        targetFile = self._getFSFullPath(PLFNGRelativePath=PLFNGRelativePath)
        
        if not os.path.isfile(targetFile): return result

        if arg == 'mod_date':
           try: result = DateTime(os.stat(targetFile)[8])
           except: result = DateTime()
        
        if arg == 'author':
           result = getMetadataElement(targetFile,
                                       "GENERAL",
                                       "source",
                                       REQUEST=self.REQUEST)
           if result==None: result='admin'
        
        if arg == 'revision':
           result = getMetadataElement(targetFile,
                                       "GENERAL",
                                       "revision",
                                       REQUEST=self.REQUEST)
           if result==None: result='1'
        
        return result


    security.declareProtected('View', 'getFileMetadata')
    def getFileMetadata(self, section="GENERAL", option="comment", rel_dir=''):
        """
        wrapper method of util.getMetadataElement() to enable page templates
        to get a specified subset of the metadata for a file or folder contained
        by the PLFNG instance.
        
        :Parameters:
            - `section`: 
            - `option`: 
            - `rel_dir`: 

        :Return:
            - the specified subset of the metadata for the item.
        """
        targetFile = self._getFSFullPath(PLFNGRelativePath=rel_dir)
        #zLOG.LOG('PLFNG', zLOG.INFO , "getFileMetadata() :: \
        #targetFile = %s" % targetFile)
        metadataText = getMetadataElement(targetFile,
                                          section,
                                          option,
                                          REQUEST=self.REQUEST)
        return metadataText


    security.declareProtected(ManagePortal, 'getFileRealPath')
    def getFileRealPath(self):
        """
        get the real (file system) path for the file
        
        :Parameters:
            - none

        :Return:
            - the file system path for the file.
        """
        return self._getFSFullPath(PLFNGRelativePath='')

    security.declareProtected('View', 'getFileSearchResultText')
    def getFileSearchResultText(self,PLFNGRelativePath, searchterm, desc_length, desc_leftPadding):
        try:
           from Products.TextIndexNG2.Registry import ConverterRegistry, RegistryException
        except:
           ConverterRegistry = None

        
        """
        get the relevent text from a PLFNG file to show in search results
        
        :Parameters:
            - PLFNGRelativePath

        :Return:
            - the relevent search results text from the PLFNG file.
        """

        this_portal = getToolByName(self, 'portal_url')
        mimetypesTool = getToolByName(this_portal, 'mimetypes_registry')
        
        source = ''
        
        targetFile = self._getFSFullPath(PLFNGRelativePath=PLFNGRelativePath)
        
        if not os.path.isfile(targetFile): return ''
        
        file = open(targetFile, 'rb')
        try: source = file.read()
        finally: file.close()

        mi = mimetypesTool.classify(data=None, filename=targetFile.lower())
        try:
           mimetype = mi.normalized()
        except:
           mimetype = mimetypesTool.defaultMimetype
        
        # this is from TextIndexNG.py
        encoding = 'iso-8859-15'
        
        #if self.use_converters:
        if ConverterRegistry:
           if mimetype is None or mimetype == 'application/octet-stream':
               mimetype, encoding = guess_content_type(obj.getId(), source)
               if not encoding:
                   encoding = self.default_encoding
   
           try: 
               converter = ConverterRegistry.get(mimetype)
           except RegistryException: 
               #LOG('textindexng', ERROR, '%s could not be converted because no converter could be found for %s' % (obj.absolute_url(1), mimetype))
               msg = '%s could not be converted because no converter could be found for %s' % (targetFile, mimetype)
               zLOG.LOG('PloneLocalFolderNG', zLOG.ERROR , msg)
               return ''
   
           if converter:
               try:
                   source, encoding = converter.convert2(source, encoding, mimetype)
               except:
                   try:
                       source = converter.convert(source)
                   except:
                       #LOG('textindexng', ERROR, '%s could not be converted' % obj.absolute_url(1), error=sys.exc_info())
                       msg = '%s could not be converted' % targetFile
                       zLOG.LOG('PloneLocalFolderNG', zLOG.ERROR , msg)
                       return ''
     
        searchterm = searchterm.split(' ')[0]
        #findIndexStart = source.find(searchterm)
        
        import re
        if searchterm.startswith('*'): searchterm = searchterm[1:]
        p = re.compile(searchterm,re.IGNORECASE)
        m = p.search(source)
        if m: findIndexStart = m.start()
        else: findIndexStart = 0

        paddedFindIndexStart = max(0,findIndexStart - desc_leftPadding)
        paddedFindIndexStop = min(len(source),paddedFindIndexStart+desc_length)
        
        if findIndexStart != -1: return source[paddedFindIndexStart:paddedFindIndexStop]
        else: return source[:desc_length]

    security.declareProtected('View', 'getFilteredContents')
    def getFilteredContents(self,rel_dir=None):
        """ list filtered content of local filesystem """

        filteredFileList = []
        filteredFolderList = []
        resultList = []

        if rel_dir == None: rel_dir = ''
        destpath = self._getFSFullPath(PLFNGRelativePath=rel_dir)

        if rel_dir == '':
           outputPrefix = ''
        else:
           outputPrefix = rel_dir.replace('\\', '/') + '/'

        fileFilter = self.createFileFilter()
        
        filteredFileList, filteredFolderList = \
            fileFilter.listFolder(folder_path=destpath)                                            

        for filename in filteredFileList:
            resultList.append(outputPrefix + filename)

        for subFolder in filteredFolderList:
            resultList.append(outputPrefix + subFolder+'/')
            new_rel_dir = os.path.join(rel_dir,subFolder)
            sub_resultList = self.getFilteredContents(rel_dir=new_rel_dir)
            resultList = resultList + sub_resultList

        return resultList

    security.declareProtected('View', 'getFilteredOutContents')
    def getFilteredOutContents(self,rel_dir=None):
        """ list filtered out content of local filesystem 
        
        TODO: Illegal files list is not supported in new FileFilter
        """

        illegalFilesList = []
        filteredOutFileList = []
        filteredOutFolderList = []

        filteredFileList = []
        filteredFolderList = []

        if rel_dir == None: rel_dir = ''
        destpath = self._getFSFullPath(PLFNGRelativePath=rel_dir)

        this_portal = getToolByName(self, 'portal_url')
        mimetypesTool = getToolByName(this_portal, 'mimetypes_registry')

    
        fileFilter = self.createFileFilter()
        
        filteredFileList = fileFilter.listFolder(folder_path=destpath,
                                                 separate_files_and_folders=False)
        trimmedFolderBasePath = os.path.normpath(destpath)

        filteredOutFileList, filteredOutFolderList = \
            fileFilter.listFolder(folder_path=destpath,
                                  separate_files_and_folders=False,
                                  inverse=True)

        filteredFileList, filteredFolderList = \
           fileFilter.listFolder(folder_path=destpath,
                                  separate_files_and_folders=False)

        for subFolder in filteredFolderList:
           new_rel_dir = os.path.join(rel_dir,subFolder)
           sub_illegalFilesList, sub_illegalFoldersList, \
              sub_filteredOutFileList, sub_filteredOutFolderList = \
              self.getFilteredOutContents(rel_dir = new_rel_dir)

           illegalFilesList = illegalFilesList + sub_illegalFilesList
           illegalFoldersList = illegalFoldersList + sub_illegalFoldersList
           filteredOutFileList = \
              filteredOutFileList + sub_filteredOutFileList
           filteredOutFolderList = \
              filteredOutFolderList + sub_filteredOutFolderList

        return illegalFilesList, illegalFoldersList, \
               filteredOutFileList, filteredOutFolderList

    security.declareProtected(CMFCorePermissions.View,
                              'getMoveActionRestrictionsVocab')
    def getMoveActionRestrictionsVocab(self):
        return DisplayList(
           (
            ('allow_local_movement_only',
             'only allow file movement within this PLFNG instance'),
            ('allow_any_movement',
             'allow movement anywhere on the file system'),
           ))
    
    security.declareProtected('View', 'getProperties')
    def getProperties(self, REQUEST=None):
        """ get the summary properties for the local filesystem directory
            for this class instance """

        FSFullPath = self._getFSFullPath(PLFNGRelativePath='')
        localfolder_props = getFolderProperties(FSFullPath)
        return localfolder_props

    security.declareProtected('View', 'get_size')
    def get_size(self):
        """ return the size of the underlying contents """

        if self.validFolder():
           trimmedFSBasePath = self._getFSBasePath()
           localfolder_props = getFolderProperties(trimmedFSBasePath)
           return localfolder_props.get('size',0)
        else:
           return 0


    security.declareProtected(AddPortalContent, 'HTMLFormUploadFile')
    def HTMLFormUploadFile(self, upload, comment, clientMD5=None):
        """ upload a file via HTML form-based file upload mechanism """

        destpath = self._getFSFullPath(PLFNGRelativePath='')
        trimmedFSBasePath = self._getFSBasePath()
        rel_dir = destpath.replace(trimmedFSBasePath, '')
        if rel_dir.startswith(os.sep): rel_dir = split(rel_dir,os.sep, 1)[1]

        # pppffff....some IE browsers apparently do NOT strip the path from
        # the filename on form submittals, so we need to do so here
        if '\\' in upload.filename:
            filename = upload.filename.split('\\')[-1]
        else:
            filename = upload.filename

        uploadFileBaseName = os.path.basename(filename)
        if self.backup_folder:
            backupdestpath = os.path.join(self.backup_folder, rel_dir)
        else:
            backupdestpath = None

        if not upload:
            self.REQUEST.RESPONSE.redirect(self.REQUEST['URL1']+\
              '/plfng_view?portal_status_message=no file was uploaded!')
            return 0

        if self.require_MD5_with_upload and not clientMD5:
            msg = 'you MUST provide the MD5 checksum for the file you want \
            to upload!'
            self.REQUEST.RESPONSE.redirect(self.REQUEST['URL1']+\
              '/plfng_view?portal_status_message=' + msg)
            return 0

        destFSfullPath = os.path.join(destpath, os.path.basename(filename))
        tmpfile = destFSfullPath + '.plfngtmp'
        f = open(tmpfile, 'wb')
        f.write(upload.read())
        f.close()
        serverMD5 = None

        newRevisionNumber = \
         self.addFile(sourceFSfullPath=tmpfile,
                     destPLFNGrelativePath=rel_dir,
                     destBasename=os.path.basename(destFSfullPath),
                     moveFile=1,
                     inputMD5=clientMD5)

        if newRevisionNumber == 0:
           os.remove(tmpfile)
           self.REQUEST.RESPONSE.redirect(self.REQUEST['URL1']+\
             '/plfng_view?portal_status_message=file rejected.')
        else:

           # ------------------------- apply metadata -------------------------
           # GENERAL:
           #   comments    - comments on the file
           #   description - (tbd)
           #   source      - userId of the source/uploader/provider
           #   language    - language code for the file contents
           #   revision    - the PLFNG centric revision # of this file
           # DIAGNOSTICS:
           #   md5         - server-validated md5 checksum of this file
           # ARCHIVEINFO:
           #   numUnpackedFiles   - # files that the packed file contains
           #   sizezUnpackedFiles - total # bytes for all of the unpacked files
           # CHANGELOG:
           #   history     - (tbd)
           # ------------------------------------------------------------------

           # GENERAL: 'comments' option
           if comment:
               setMetadata(destFSfullPath,
                           section="GENERAL",
                           option="comment",
                           value=comment)

           # GENERAL: 'source' option
           portal = getToolByName(self, 'portal_url').getPortalObject()
           portal_membership = getToolByName(portal, 'portal_membership')
           if portal_membership.isAnonymousUser():
               creator = 'anonymous'
           else:
               creator = portal_membership.getAuthenticatedMember().getId()
           setMetadata(destFSfullPath,
                       section="GENERAL",
                       option="source",
                       value=creator)

           # GENERAL: 'revision' option
           setMetadata(destFSfullPath,
                       section="GENERAL",
                       option="revision",
                       value=newRevisionNumber)

           # DIAGNOSTICS: 'md5' option
           if clientMD5:
               setMetadata(destFSfullPath,
                           section="DIAGNOSTICS",
                           option="md5",
                           value=clientMD5)

           # if .zip file, set ARCHIVEINFO metadata
           if self.mimetypes_registry.classify(data=None, \
             filename=filename.lower()) == 'application/zip':
               setZipInfoMetadata(destFSfullPath)

           self.REQUEST.RESPONSE.redirect(self.REQUEST['URL1']+\
             '/plfng_view?portal_status_message=file added.')

    security.declareProtected('View', 'listFolderContents')
    def listFolderContents(self,  spec=None, contentFilter=None,
                           suppressHiddenFiles=0, REQUEST=None, RESPONSE=None):
        """ list content of local filesystem 
        
        @param spec = ?
        @param contentFilter = ?
        @param suppressHiddenFiles = ?
        """

        filteredFileList = []
        filteredFolderList = []

        destpath = self._getFSFullPath(PLFNGRelativePath='')

        trimmedFSBasePath = self._getFSBasePath()

        relURL = destpath.replace(trimmedFSBasePath, '').replace('\\','/')

        #if relURL.startswith('/'): relURL = relURL[1:]
        relURL = relURL+'/'
        
        if not relURL.startswith('/'): 
            relURL = '/'+relURL
        
        this_portal = getToolByName(self, 'portal_url')
        mimetypesTool = getToolByName(this_portal, 'mimetypes_registry')
        
        # List files of the folder using filtering rules
        fileFilter = self.createFileFilter()
        filteredFileList, filteredFolderList = fileFilter.listFolder(destpath)

        proxyObjectsList = []

        for f in filteredFolderList:

            FSfullPathFolderName = os.path.join(destpath, f)
            P = FileProxy(id=f,
                          filepath=FSfullPathFolderName,
                          fullname=f,
                          properties=None)

            P.setIconPath('folder_icon.gif')
            
            P.setAbsoluteURL(self.absolute_url() + "/" + f)
            P.setMimeType('folder')
            if os.path.exists(FSfullPathFolderName + '.metadata'):
                try:
                  P.setComment(getMetadataElement(FSfullPathFolderName,
                                                  section="GENERAL",
                                                  option="comment",
                                                  REQUEST=self.REQUEST))
                except:
                  P.setComment('')
                try:
                  P.setTitle(getMetadataElement(FSfullPathFolderName,
                                                section="GENERAL",
                                                option="title",
                                                REQUEST=self.REQUEST))
                except:
                  P.setTitle('')

            else:
                P.setComment('')
                P.setTitle('')

            proxyObjectsList.append(P)

        for file in filteredFileList:

            destpath = self._getFSFullPath(PLFNGRelativePath='')
            trimmedFSBasePath = self._getFSBasePath()
            #rel_dir = destpath.replace(trimmedFSBasePath, '')
            rel_dir = destpath.replace(trimmedFSBasePath, '').replace('\\','/')
            if rel_dir.startswith('/'): rel_dir = rel_dir[1:]

            _proxy = self._createProxy(file,
                                       rel_dir=rel_dir,
                                       destination=destpath)

            proxyObjectsList.append(_proxy)

        return proxyObjectsList


    security.declareProtected(ModifyPortalContent, 'moveFile')
    def moveFile(self, sourceFSfullPath, destFSfullPath):
        """ move a file. destFSfullPath can be a file or folder """

        #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO, "moveFile(%s,%s)" % \
        #(sourceFSfullPath,destFSfullPath))
        
        
        
        # 1st, make sure that a file to move was specified

        if not os.path.isfile(sourceFSfullPath):
           zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "moveFile() :: Error! \
           sourceFSfullPath = %s not a file" % sourceFSfullPath )
           return 0

        # next, make sure destination path is valid
        if os.path.isfile(destFSfullPath):
            targetFSFolder = os.path.dirname(destFSfullPath)
            targetFSBaseName  = os.path.basename(destFSfullPath)
        else:
            targetFSFolder = destFSfullPath
            targetFSBaseName  = os.path.basename(sourceFSfullPath)
        if targetFSFolder.find('..') > -1 :
            msg="moveFile aborted: illegal destination name: " + targetFSFolder
            zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , msg )
            return 0
        
        fileDrive, filePath = os.path.splitdrive(os.path.normpath(targetFSFolder))
        
        if len(filePath) > 1:
            pathItemList = split(filePath[1:],os.sep)
        else:
            pathItemList = None
        
        for pathItem in pathItemList:
            #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "moveFile() :: pathItem = %s" % pathItem)
            checkValidIdResult = checkValidId(pathItem)
            if checkValidIdResult != 1:
                zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "moveFile() :: \
                  Error! destination path (%s) invalid: %s.  pathItem = %s" % \
                  (targetFSFolder,checkValidIdResult,pathItem))
                return 0
        
        trimmedFSBasePath = self._getFSBasePath() 
        
        # find out if the file being moved is contained by this PLFNG instance
        if sourceFSfullPath.startswith(trimmedFSBasePath): 
            srcFileIsLocal = 1
        else:
            srcFileIsLocal = 0
        
        # find out if the destination location for the file being moved is
        # local to this PLFNG instance
        if destFSfullPath.startswith(trimmedFSBasePath): 
            dstFileIsLocal = 1
        else:
            dstFileIsLocal = 0
        
        # mpg: Important!!!! this is not finished......the following
        # code is only a very crude temporary patch.
        
        shutil.move(sourceFSfullPath, destFSfullPath)
        
        #try:
        #    return 1
        #except:
        #    zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "moveFile() :: \
        #       shutil.move(%s,%s) raised Exception" % (sourceFSfullPath, destFSfullPath) )       
        #       return 0  
        # if file movement only local to this PLFNG instance is allowed,
        # make sure that the source and destination paths begin with
        # the base file system path of this PLFNG instance
         
        #if self.item_move_rules == 'allow_local_movement_only':
        #    if not srcFileIsLocal:
        #        zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "moveFile() :: \
        #         Error! source path (%s) not local to the base path (%s) \
        #         of this PLFNG folder" % (sourceFSfullPath,trimmedFSBasePath))
        #        return 0
            
        #    if not dstFileIsLocal:
        #        zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "moveFile() :: \
        #         Error! destination path (%s) not local to the base path (%s) \
        #         of this PLFNG folder" % (targetFSFolder,trimmedFSBasePath))
        #        return 0    
        
            
        #elif self.item_move_rules == 'allow_any_movement':

        # if the destination path is within the FS path of this PLFNG instance,
        # use the PLFNG instances' addFile() method to move the file so that
        # metadata and backup aspects are taken care of automatically.
        #if dstFileIsLocal:
        #    destPLFNGrelativePath = targetFSFolder.replace(trimmedFSBasePath, '')
        #    destPLFNGrelativePath = os.path.dirname(destPLFNGrelativePath)
        #    destPLFNGrelativePath = destPLFNGrelativePath.replace('\\','/')
        #    if destPLFNGrelativePath == '/': destPLFNGrelativePath = ''
        #    elif destPLFNGrelativePath.startswith('/'): rel_dir = rel_dir[1:]
        #    zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "moveFile() :: \
        #         calling addFile(%s,%s,%s,%d,%s)" % (sourceFSfullPath, destPLFNGrelativePath,
        #                 targetFSBaseName,1, None) )
            
        #    self.addFile(sourceFSfullPath, 
        #                 destPLFNGrelativePath=destPLFNGrelativePath,
        #                 destBasename=targetFSBaseName,
        #                 moveFile=1, 
        #                 inputMD5=None)
        
        #else:  
        #    if not srcFileIsLocal: shutil.move(sourceFSfullPath, destFSfullPath)
        #    try:
        #       # if the source path is within the FS path of this PLFNG
        #       # instance, use shutil.copy() with the PLFNG instances' 
        #       # deleteFile() method to move the file so that metadata and
        #       # backup aspects are taken care of automatically.
        #       if srcFileIsLocal:
        #          shutil.copy(sourceFSfullPath, destFSfullPath)
        #          self.deleteFile(sourceFSfullPath,None,None)
        #       #mpg fix
        #       #else:
        #       #   shutil.move(sourceFSfullPath, destFSfullPath)
        #    except:
        #       zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "moveFile() :: \
        #       Error! could not move file" )
        #       return 0



    security.declareProtected(AddPortalContent, 'packFiles')
    def packFiles(self, plfngRelativeDir, filesList):
        """ pack a set of files """
        
        zipFiles(self, plfngRelativeDir, filesList)

    security.declareProtected(ModifyPortalContent, 'setFileMetadata')
    def setFileMetadata(self, section, option, newvalue, filePLFNGrelativePathName=''):
        """ set the metadata for the file """
        result = 0
        #zLOG.LOG('PLFNG', zLOG.INFO , "setFileMetadata() :: \
        #section=%s option=%s newvalue=%s" % (section, option, newvalue))

        targetFile = self._getFSFullPath(PLFNGRelativePath=filePLFNGrelativePathName)

        if section and option and newvalue:
            if len(section)==len(option)==len(newvalue):
                tmpres = len(section) > 0  # initialize to True
                for i in range(0,len(section)):
                    tmpres = tmpres and setMetadata(targetFile,
                                                    section[i],
                                                    option[i],
                                                    newvalue[i])
                # return True when all matadatas are successfully set
                result = tmpres

        if result == 1:
            if self.REQUEST:
               self.REQUEST.RESPONSE.redirect(self.REQUEST['URL1']+\
                  '/plfng_editMetadata?portal_status_message=file metadata updated.')
            else:
               return result      
        else:
            msg = 'Error updating file metadata.'
            if self.REQUEST:
               self.REQUEST.RESPONSE.redirect(self.REQUEST['URL1']+\
                  '/plfng_editMetadata?portal_status_message=' + msg)
            else:
               return result 

    security.declareProtected('View', 'showFile')
    def showFile(self, destpath, REQUEST, RESPONSE):
        """ view file """
        
        print "showFile:" + destpath

        mi = self.mimetypes_registry.classify(data=None,
                                              filename=destpath.lower())
        try:
           item_mime_type = mi.normalized()
        except:
           item_mime_type = self.mimetypes_registry.defaultMimetype

        RESPONSE.setHeader('content-type', item_mime_type)
        RESPONSE.setHeader('content-length', str(os.stat(destpath)[6]))
        if REQUEST.get('action', '') == 'download':
            # TODO: Does this handle spaces in filenames
            REQUEST.RESPONSE.setHeader('content-disposition',
                                       'attachment; filename=%s'
                                        % os.path.basename(destpath))

        _onFileAccess(self,FSFullPath=destpath)
        
        fiter = None
        if STREAM_SUPPORT:
            fiter = filestream_iterator(destpath, 'rb')
        else:
            fp = open(destpath, 'rb')
            while 1:
                data = fp.read(32768)
                if data:
                    RESPONSE.write(data)
                else:
                    break
            fp.close()

        return fiter


    security.declareProtected(AddPortalContent, 'unpackFile')
    def unpackFile(self, plfngRelativeDir, FSpackedFile, output='redir'):
        """ unpack a file """
        
        REQUEST = self.REQUEST
        if REQUEST: RESPONSE = self.REQUEST.RESPONSE
        else: RESPONSE = None
        
        #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO, "unpackFile(%s,%s,%s,%s)" % \
        #  (plfngRelativeDir, FSpackedFile, REQUEST, RESPONSE))

        # for now, unzip is the only type of unpacking implemented
        
        if not os.path.exists(FSpackedFile):
            return 0
            
        #print "found FSpackedFile = %s" % FSpackedFile

        # 1st, make sure the file is an unpackable type
        if not self.mimetypes_registry.classify(data=None,
                                                filename=FSpackedFile.lower())\
                                                == 'application/zip':
            if output=='redir'and REQUEST and RESPONSE:
                RESPONSE.redirect(REQUEST['URL1']+\
                   '/plfng_view?portal_status_message=file cannot be unpacked \
                   (not a recognized packed file type).')
            return 0
        # then, make sure the file unpacking property is set
        elif not self.allow_file_unpacking:
            msg = 'file unpacking is not allowed here.'
            if output=='redir'and REQUEST and RESPONSE:
                RESPONSE.redirect(REQUEST['URL1']+\
                   '/plfng_view?portal_status_message=' + msg)
            return 0
        # then, check that file unpacking will not violate any quota-limits
        elif int(self.quota_maxbytes) != 0:
            # traverse up the acquisition tree looking for first container with
            # a non-zero 'quota_maxbytes' attribute.  If such a container is
            # found, find out the total number of bytes used by the contents of
            # this container in order to determine if the addition of the
            # unpacked contents of the file will exceed quota_maxbytes --in
            # which case do not carry out the unpack operation.

            max_allowed_bytes = 0

            for parent in self.aq_chain[1:]:
               if hasattr(parent, "quota_maxbytes"):
                  max_allowed_bytes = int(getattr(parent, "quota_maxbytes"))
                  if max_allowed_bytes > 0:
                     usedBytes = determine_bytes_usage(parent)
                     break

            unpackedSizeStr = getMetadataElement(FSpackedFile,
                                                 section="ARCHIVEINFO",
                                                 option="unpacked_size",
                                                 REQUEST=self.REQUEST)
            if unpackedSizeStr:
               try:
                  unpackedSize = int(unpackedSizeStr.split('.')[0])
               except:
                  unpackedSize = 0
            else:
               try:
                  setZipInfoMetadata(FSpackedFile)
                  unpackedSizeStr = getMetadataElement(FSpackedFile,
                                                       section="ARCHIVEINFO",
                                                       option="unpacked_size",
                                                       REQUEST=self.REQUEST)
                  try:
                  	unpackedSize = int(unpackedSizeStr.split('.')[0])
                  except:
                  	unpackedSize = 0

                  #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
                  #"unpackFile() :: unpackedSize=%d" % unpackedSize )

               except:
                  msg = 'file could not be unpacked (not a valid file?!).'
                  if output=='redir'and REQUEST and RESPONSE:
                      RESPONSE.redirect(REQUEST['URL1']+\
                         '/plfng_view?portal_status_message=' + msg)
                  return 0
            if max_allowed_bytes > 0 and \
             (unpackedSize + usedBytes) > max_allowed_bytes:
               msg = 'file cannot be unpacked as doing so would violate \
               quota limit.'
               zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "unpackFile() :: \
               unpack rejected! unpackedSize (%s) + usedBytes(%s) > \
               max_allowed_bytes(%s)" % \
               (unpackedSize,usedBytes, max_allowed_bytes) )
               if output=='redir'and REQUEST and RESPONSE:
                   REQUEST.RESPONSE.redirect(REQUEST['URL1']+\
                      '/plfng_view?portal_status_message=' + msg)
               return 0
        # if we made it to this point, unpack the file
        if self.backup_folder:
            backupdestpath = os.path.join(self.backup_folder, plfngRelativeDir)
        else:
            backupdestpath = None
            
        numFilesUnpacked,numFilesSkipped = \
           upzipFile(self, plfngRelativeDir, FSpackedFile)
        if numFilesUnpacked > 0:
            if output=='redir'and REQUEST and RESPONSE:
                RESPONSE.redirect(REQUEST['URL1']+\
                  '/plfng_view?portal_status_message=unpacked %d files, failed to unpack %d files.' %(numFilesUnpacked,numFilesSkipped))
              
            if numFilesSkipped == 0: return 1
            else: return 0
        else:
            if output=='redir'and REQUEST and RESPONSE:
                RESPONSE.redirect(REQUEST['URL1']+\
                   '/plfng_view?portal_status_message=failed to unpack all %d files.' % numFilesSkipped)
            return 0

    security.declareProtected(ModifyPortalContent, 'updateMD5Metadatum')
    def updateMD5Metadatum(self,PLFNGrelativePath='',mode='',output='redir'):
        """ update (or test) the MD5 metadatum for a file 

        :Parameters:
            - `PLFNGrelativePath`: if PLFNGrelativePath is provided, it will be
               used by _getFSFullPath() to determine which target file this 
               method will operate on. Otherwise, _getFSFullPath() will use the
               '_e' field of self.REQUEST  to determine the target file.
            - `mode`: specify one of the following operation modes for this method
            	- 'testonly' -> the MD5 metadatum for the file will only be tested
            	- 'MD5only' -> only the current MD5 for the file will be generated
            	- 'update' ->  the MD5 metadatum for the file will be updated
            - `output`: specify the output/return behavior for this method
            	- 'redir' -> the textual result of this method is provided through
            	  portal_status_message via plfng_editMetadata 
            	- otherwise, return either the MD5 string for the file or an integer 
            	  value per the following table: 

        :Return:
            - refer to following table:

        +----------+----------+---------+-------------------------------------+
        | existing |parameters          |                                     |
        | metadata +----------+---------+       output/return behavior        |
        | valid?   |   mode   |  output |                                     |
        +==========+==========+=========+=====================================+
        |   yes    |'testonly'| 'redir' | redirect: 'MD5 valid'               |
        +----------+----------+---------+-------------------------------------+
        |   no     |'testonly'| 'redir' | redirect: 'MD5 invalid warning'     |
        +----------+----------+---------+-------------------------------------+
        |   yes    |'testonly'|         |             -1                      |
        +----------+----------+---------+-------------------------------------+
        |   no     |'testonly'|         |              0                      |
        +----------+----------+---------+-------------------------------------+
        |   yes    | 'update' |         |             -1                      |
        +----------+----------+---------+-------------------------------------+
        |   no     | 'update' |         |              1 **                   |
        +----------+----------+---------+-------------------------------------+ 
        |   yes    | 'update' | 'redir' |redirect: 'MD5 valid, no update'     |
        +----------+----------+---------+-------------------------------------+
        |   no     | 'update' | 'redir' |redirect: 'MD5 metadata updated'     |
        +----------+----------+---------+-------------------------------------+
        |  yes     | 'MD5only'| 'redir' |redirect: 'file MD5 is <md5>'        |
        +----------+----------+---------+-------------------------------------+
        |   no     | 'MD5only'| 'redir' |redirect: 'file MD5 is <md5>'        |
        +----------+----------+---------+-------------------------------------+
        |  yes     | 'MD5only'|         |           <md5>                     |
        +----------+----------+---------+-------------------------------------+  
        |   no     | 'MD5only'|         |           <md5>                     |
        +----------+----------+---------+-------------------------------------+        
        
        ** return value will be 1 only if setMetadata() succeeds; otherwise 
        0 will be returned
        
        """

        result = 0
        section = 'DIAGNOSTICS'
        option = 'md5'

        targetFile = self._getFSFullPath(PLFNGRelativePath=PLFNGrelativePath)
        
        #msg = "updateMD5Metadatum() :: rel_dir=%s mode=%s output=%s target=%s"\
        #       % (PLFNGrelativePath, mode, output, targetFile)
        #zLOG.LOG('PLFNG', zLOG.INFO , msg)

        if not os.path.isfile(targetFile):
            #raise ValueError('not a file: %s' % PLFNGrelativePath)
            return None

        metadataMD5 = getMetadataElement(targetFile,
                                         section,
                                         option,
                                         REQUEST=self.REQUEST)
        start_time = DateTime()
        fileMD5 = generate_md5(targetFile,self.external_syscall_md5)
        stop_time = DateTime()
        elapsed_time = (stop_time - start_time)*86400.0

        if metadataMD5 == fileMD5:
            result = -1
            if mode == '' or mode=='testonly':
                msg = 'MD5 metadata is valid. (MD5 computation took '+ \
                      str(elapsed_time)[:6] + ' seconds).'
            elif mode == 'MD5only':
                result = fileMD5
                msg='current file MD5 is: ' + fileMD5
            elif mode =='update':
                msg = 'no need to update MD5 metadata (it is valid).'
        else:
            if mode == '' or mode == 'testonly':
                result = 0
                msg='WARNING: MD5 metadata does NOT match current file MD5 !!!'
            elif mode == 'MD5only':
                result = fileMD5
                msg='current file MD5 is: ' + fileMD5
            elif mode =='update':
                result = setMetadata(targetFile, section, option, fileMD5)
                if result == 1:
                     msg = 'MD5 metadata updated.'
                else:
                     msg = 'ERROR: MD5 metadata could not be updated !!!'

        if output == 'redir':
           self.REQUEST.RESPONSE.redirect(self.REQUEST['URL1']+\
           '/plfng_editMetadata?portal_status_message=' + msg)
        else:
           return result


    security.declareProtected('View', 'validFolder')
    def validFolder(self):
        """ determine if the requested folder path is legal and exists """

        # Note: we do NOT want _getFSFullPath() to perform a redirect action
        # if the PLFNG instance folder path is invalid.  For now, this is not
        # a problem as the redirection logic in _getFSFullPath() doesnt appear
        # to work, but if it does start working, we will need to pass a control
        # param into it to tell it whether or not to redirect on error.

        try:
           rtn = self._getFSFullPath(PLFNGRelativePath='')
           return 1
        except:
           return 0


    security.declareProtected(View, 'view')
    def view(self, REQUEST, RESPONSE):
        """
        Invokes the default view.
        
        :Parameters:
            - `REQUEST`: the standard HTML request data structure.
            - `RESPONSE`: the standard HTML response data structure.

        :Return:
            - the return value of the invoked default view.
        """

        return self.__call__(REQUEST, RESPONSE)


def modify_fti(fti):
    """ Patch factory type information from inherited data
    """
    
    # hide unnecessary tabs (usability enhancement)
    for a in fti['actions']:
        if a['id'] in ('references',):
            a['visible'] = 0
    fti['filter_content_types'] = 1
    fti['allowed_content_types'] = ()
    return fti

registerType(PloneLocalFolderNG, PROJECTNAME)
