"""
    Functional tests for navtree related PloneLocalFolderNG properties
    
    Copyright 2006 Mikko Ohtamaa <miohtama@mail.student.oulu.fi>
    
    Requires:
        - Plone 2.1

"""

__author__  = 'Mikko Ohtamaa <snaapy@gmail.com>'
__docformat__ = 'restructuredtext'

# Python imports
import os, sys
import time
from StringIO import StringIO
import urllib

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

# Zope Imports
from Testing import ZopeTestCase

# Plone imports
from Products.Archetypes.atapi import *
from Products.Archetypes.tests.attestcase import default_user
from Products.Archetypes.tests.attestcase import HAS_PLONE21
from Products.Archetypes.tests.atsitetestcase import portal_owner
from Products.Archetypes.tests.atsitetestcase import ATFunctionalSiteTestCase
from Products.Archetypes.tests.utils import DummySessionDataManager
from Products.Archetypes.tests.utils import makeContent
from Products.CMFPlone.tests.PloneTestCase import PloneTestCase
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import *

# Local imports
from Products.PloneLocalFolderNG import initialize
from Products.PloneLocalFolderNG.CatalogProxy import CatalogProxy
from Products.PloneLocalFolderNG.tests.test_plfng import SAMPLES_PATH
from PLFNGFunctionalTestCase import PLFNGFunctionalTestCase
from PLFNGFunctionalTestCase import print_sitemap

# Run __init__.py's initialize(context) for products used in this test case
# This registers types and other stuff
ZopeTestCase.installProduct('ZCatalog')
ZopeTestCase.installProduct('MimetypesRegistry')
ZopeTestCase.installProduct('Archetypes')
ZopeTestCase.installProduct('PloneLocalFolderNG')
ZopeTestCase.installProduct('PortalTransforms')

class TestNavtree(PLFNGFunctionalTestCase):
    """ Functional tests for cataloging and navtree related functionality
    
    Tests for:
        - Bread crumbs
        - Navtree 
        - Sitemap
    """

    def afterSetUp(self):
        """ Do PloneLocalFolderNG navtree specific common test setup
        
        - Setup PloneLocalFolderNG instance into test environment        
        """
        PLFNGFunctionalTestCase.afterSetUp(self)

        plone_tool = self.portal.plone_utils    
                
        # Check that our CatalogProxy can be found from types
        # (quick installer has succeeded and this is requirement for 
        # site map generation)
        if not CatalogProxy.portal_type in plone_tool.typesToList():
            self.fail("CatalogProxy not found in plone_utils types - navtree doesn't function propeply. Types:" + str(plone_tool.typesToList()))
                
        url = self.portal.localfolder.absolute_url(relative=True) + "/plfng_catalog?action=catalog_now"
        
        # Catalog files so that file paths are added to portal_catalog
        response = self.publish(url, self.basic_auth)         
        self.assertStatusEqual(response.getStatus(), 200) # OK   
        
        self.failUnless("<b>total # files cataloged = </b>" in response.getBody(), "Cataloging sample files failed")
            
                                                                   
    def test_breadcrubs(self):
        """ Test that we get proper bread crumbs for global_pathbar.pt 
        
            - First test that bread crumbs are propeply created
            - Then check that HTML page is ok 
        """

        plone_tool = self.portal.plone_utils    
        # save request object so it is not reset after publish
        #response = self.publish(self.folder_path + "/localfolder/subfolder/sample.jpg",
        #                        self.basic_auth)                    
        
        #print "resp:" + str(response)
                
        request = self.app.REQUEST
        
        # fake REQUEST["_e"], meaning current file path
        # PloneLocalFolderNG.__bobo_traverse__ normally generates this
        # but we cannot directly call publish with a specific path
        request["_e"] = [ "subfolder", "sample.jpg" ]
        
        crumbs = self.portal.plone_utils.createBreadCrumbs(self.portal.localfolder)
        print "crumbs:" + str(crumbs)
        
        self.failUnless(len(crumbs) == 3, "bread crumbs not generated correctly")
        
        self.failUnless(crumbs[2]["absolute_url"] == 'http://nohost/plone/localfolder/subfolder/sample.jpg')
        self.failUnless(crumbs[1]["absolute_url"] == 'http://nohost/plone/localfolder/subfolder')
        self.failUnless(crumbs[0]["absolute_url"] == 'http://nohost/plone/localfolder')
        
        # Test page rendering        
        response = self.publish(self.folder_path +
            "/localfolder/subfolder/sample.jpg/plfng_view",
            self.basic_auth) 
        
        # TODO: See that global_pathbar is rendered correctly in response      
                                
    def test_sitemap(self):
        """ Check that local mounted files are presented in sitemap 
        
        TODO: This is not real functional tests since we are not veryfying
              sitemap from portal/sitemap output HTML
              
        Things done:
            - First create sitemap manually so that errors are easier to track down
              (stdout stack traces are generated)
            - Then check that a valid sitemap page is generated
        """
        
        portal = self.portal
        
        # Check that our CatalogProxy can be found from types
        plone_tool = self.portal.plone_utils
        
        # set sitemap depth to infinite so that all sample files
        # are in recurse depth range
        ntp = portal.portal_properties.navtree_properties
        ntp._setPropValue('sitemapDepth', 999)
                
        plone_tool = getToolByName(portal, "plone_utils")
        sitemap = plone_tool.createNavTree(portal, sitemap = True)        
        
        all_found = False
        
        # For debugging purposes
        # print_sitemap(sitemap)
        
        # See that all level of files from our SampleFiles folder are present
        for child in sitemap["children"]:
            if child["path"] == "/plone/localfolder":                        
                for lf_item in child["children"]:
                    if lf_item["path"] == "/plone/localfolder/subfolder":
                        # Files don't appear in navtree unless this is set
                        self.failUnless(lf_item["show_children"] == True, "Cataloged folders don't show children")
                        for lf_item3 in lf_item["children"]:
                            if lf_item3["path"] == "/plone/localfolder/subfolder/sample.jpg":
                                all_found = True
                                test_item = lf_item3
                            
                            
                                        
        if not all_found:
            print_sitemap(sitemap)
            self.fail("Not all sample files were present in sitemap. Sitemap:" + str(sitemap))
            
        # Confirm that sitemap page is rendered
        response = self.publish(self.folder_path +
            "/sitemap",
            self.basic_auth)         
        
        self.assertStatusEqual(response.getStatus(), 200) # OK               
        
def test_suite():
    import unittest
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestNavtree))
    return suite

if __name__ == '__main__':
    # modify these settings to reduce output
    framework(descriptions=False, verbosity=False)
    